use Test::More tests => 8;
use v5.14;

use_ok( 'UAV::Pilot::SDL' );
use_ok( 'UAV::Pilot::SDL::Events' );
use_ok( 'UAV::Pilot::SDL::Joystick' );
use_ok( 'UAV::Pilot::SDL::NavFeeder' );
use_ok( 'UAV::Pilot::SDL::Video' );
use_ok( 'UAV::Pilot::SDL::VideoOverlay' );
use_ok( 'UAV::Pilot::SDL::VideoOverlay::Reticle' );
use_ok( 'UAV::Pilot::SDL::Window' );
