package UAV::Pilot::SDL;
use v5.14;
use warnings;
use Moose;
use namespace::autoclean;

our $VERSION = 0.1;


no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__


=head1 NAME

    UAV::Pilot::SDL

=head1 DESCRIPTION

SDL extensions for C<UAV::Pilot>.

=cut
